@@ -1,10 +1,10 @@
 package Net::DNS::RR::HIP;
 
 #
-# $Id: HIP.pm 1382 2015-08-26 20:39:33Z willem $
+# $Id: HIP.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -46,7 +46,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless $self->{hitbin};
+	return '' unless defined $self->{hitbin};
 	my $hit = $self->hitbin;
 	my $key = $self->keybin;
 	my $nos = pack 'C2n a* a*', length($hit), $self->pkalgorithm, length($key), $hit, $key;
@@ -57,17 +57,17 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{hitbin};
+	return '' unless defined $self->{hitbin};
 	my $base64 = encode_base64( $self->keybin, '' );
 	my @server = map $_->string, @{$self->{servers}};
-	my @rdata = $self->pkalgorithm, $self->hit, $base64, @server;
+	my @rdata = ( $self->pkalgorithm, $self->hit, $base64, @server );
 }
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
-	$self->$_(shift) for qw(pkalgorithm hit key);
+	foreach (qw(pkalgorithm hit key)) { $self->$_(shift) }
 	$self->servers(@_);
 }
 
@@ -176,7 +176,6 @@ The binary representation of the host identity tag.
 
 =head2 pubkey
 
-
 =head2 key
 
     $key = $rr->key;
